//
//  CertificateAuthority.java
//

package hotTip;

//  AWT/Swing
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

//  Java
import java.util.*;
import java.math.BigInteger;

// socket
import java.net.*;
import java.io.*;
import java.net.*;
import javax.net.ssl.*;

//  Crypto
import java.security.*;
import java.security.cert.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import javax.security.auth.x500.*;

//IAIK Classes required
import iaik.x509.*;
import iaik.asn1.structures.AlgorithmID;

public class CertificateAuthority {

    //  Failure codes
    public static final int SUCCESS = 0;
    public static final int KEYSTORE_FILE_NOT_FOUND = 1;
    public static final int PERMISSIONS_FILE_NOT_FOUND = 2;
    public static final int PERMISSIONS_FILE_TAMPERED = 3;
    public static final int ERROR = 4;
    
    public static final int MAX_NUM_CLIS = 512;
    public static final int NUM_SALT_BYTES = 4;
    
    //  The GUI
    CertificateAuthorityPanel _panel;
    CertificateAuthorityActivity _activityPanel;
    CardLayout _layout;
    JFrame _appFrame;
    
    CertificateAuthorityThread _thread;
    
    //  Port number to listen on
    private int _portNum;
    
    //  Data structures to hold the authentication 
    //  information read from the file
    //  ............
    //  ............
    //  < you are free to make these whatever you like... >
    
    //
    Hashtable clientsToSaltedHashedPwds = null;    
    
    //
    // stores: clientName-to-Integer mapping
    //
    Hashtable clientsToIds = null;
    
    //
    // indexed by the integer value stored in clientsToIds
    //
    byte[][] clientSalts = null;
    
    
    /********************************************************************
     *
     *
     *
     ********************************************************************/
     public CertificateAuthority() throws Exception {
	 
	 _panel = null;
	 _activityPanel = null;
	 _layout = null;
	 _appFrame = null;
	 
	 clientsToSaltedHashedPwds = new Hashtable();
	 clientsToIds = new Hashtable();
	 
	 clientSalts = new byte[MAX_NUM_CLIS][NUM_SALT_BYTES];
	 
	 try {
	     initialize(); }
	 
	 catch (Exception e) {
	   System.err.println("CA error: " + e.getMessage());
	   e.printStackTrace(); }
	 
	 _layout.show(_appFrame.getContentPane(), "CAPanel");
	 
     }    

    /********************************************************************
     *
     *
     *
     ********************************************************************/
    private void initialize() throws Exception {
	
	_appFrame = new JFrame("Certificate Authority");
	_layout = new CardLayout();
	
	_appFrame.getContentPane().setLayout(_layout);
	_panel = new CertificateAuthorityPanel(this);
	_appFrame.getContentPane().add(_panel, "CAPanel");
	
	_activityPanel = new CertificateAuthorityActivity(this);
	_appFrame.getContentPane().add(_activityPanel, "ActivityPanel");       
	
	_appFrame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    quit();
		}
	    });
    }

    /********************************************************************
     *
     *
     *
     ********************************************************************/    
    public void run() {
	_appFrame.pack();
	_appFrame.setVisible(true);
    }
    
    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public void quit() {
	
	System.exit(0);
    }
    

    /********************************************************************
     *
     * startup CA
     *
     ********************************************************************/
    public int startup(String _ksFileName,
		       char[] _privateKeyPass,
		       String _permissionsFileName,
		       char[] _permissionsFilePass,
		       int _caPort)  {

	_portNum = _caPort;
	
	//
	// Decrypt & verify the permissions file
	
	// Populate the hashtables

	// Generate appropriate client salt bytes and add them to 
	// clientSalts array	
	
	//  Read the CA keystore (i.e. its private key)
		
	//  Failure codes to return are defined on the top	
	
	_layout.show(_appFrame.getContentPane(), "ActivityPanel");
	
	_thread = new CertificateAuthorityThread(this);
	_thread.start();
	
	return CertificateAuthority.SUCCESS;
	
    }

    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public int getPortNumber() {
    
      return _portNum;
    }
  
    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public JTextArea getOutputArea() {
    
      return _activityPanel.getOutputArea();
    }

    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public Hashtable getClientsToHshdSltdPwds( ) {

	return clientsToSaltedHashedPwds;

    }

    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public Hashtable getClientsToIds( ) {

	return clientsToIds;

    }

    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public byte[][] getClientSalts( ) {

	return clientSalts;

    }    
    
    /********************************************************************
     *
     *
     *
     ********************************************************************/
    public static void main(String[] args) throws Exception {

	Security.addProvider(new com.sun.crypto.provider.SunJCE());
	
	CertificateAuthority ca = new CertificateAuthority();
	ca.run();
    }
}
